/*
	DMenuCmd.cc

	Will Eighteen, 2000: will@opaline.demon.co.uk             

	This program is free software; you can redistribute it and/or modify       
	it under the terms of the GNU General Public License as published by       
	the Free Software Foundation; either version 2 of the License, or          
	(at your option) any later version.                                        

	This program is distributed in the hope that it will be useful,            
	but WITHOUT ANY WARRANTY; without even the implied warranty of             
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              
	GNU General Public License for more details.                               

	You should have received a copy of the GNU General Public License          
	along with this program; if not, write to the Free Software                
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                  

	See COPYING for full details.

	From
	Menu.cc: simple Menu widget component - Athena widget set
	Will 18, 1998

	pillaged more or less wholesale from 'xcommand.c' by Chris. D.
	Peterson for X Consortium. Copy via Curtin Univ. Tech., Perth,
	W. Australia, course cg252-502 tutorials. Cheers!
	Xi Eighteen 1998 - overloaded constructor and much invaluable help.
*/


#include <Xm/Protocols.h>
#include "DMenuCmd.h"
#include "Application.h"
#include "MessageDialogManager.h"
#include <stdarg.h>
#include <Xm/RowColumn.h>
#include <Xm/PushB.h>
#include <Xm/Label.h>


DMenuCmd::DMenuCmd ( const char *name, int active, Boolean quitOnSelect, DMenuCmdCallback callback,
					 void *clientData, char *menuLabelText, ... ) : NoUndoCmd ( name, active )
{
	_fixedString = TRUE;
	strncpy ( _name, name, sizeof ( _name ) - 1 );
	_name [  sizeof ( _name ) - 1 ] = 0;
	_callback = callback;
	_clientData = clientData;
	_menuItemList[0] = 0;
	_menuLabelText = menuLabelText;
	_charArraySize = 0;
	_quitOnSelect = quitOnSelect;
	_popupRealised = False;

	char *menuItemText;
	va_list menuList;
	_numMenuItems = 0;

	va_start ( menuList, menuLabelText );

	while ( va_arg ( menuList, char * ) )
		_numMenuItems++;	// copy this to storage + delete fn!

	va_end ( menuList );

	va_start ( menuList, menuLabelText );
	menuItemText = va_arg ( menuList, char * );

	for ( int n = 1; menuItemText; n++ )
	{
		if ( ( strlen ( _menuItemList ) + strlen ( menuItemText ) ) >= MENUSTORAGESPACE )
		{
			char msg[1024];
			snprintf ( msg, sizeof ( msg ), "Menu item space required exceeds storage space\n\
%d allocated in DMenuCmd.h", MENUSTORAGESPACE );
			msg [ sizeof ( msg ) - 1] = 0;
			messageDialogManager->post ( msg );
		}
		else
		{
			if ( ( int ) strlen ( menuItemText ) > _charArraySize )
				_charArraySize = strlen ( menuItemText );
	
			strncat ( _menuItemList, menuItemText, sizeof ( _menuItemList ) - strlen ( _menuItemList ) - 1 );
			strncat ( _menuItemList, ",", sizeof ( _menuItemList ) - strlen ( _menuItemList ) - 1 );
		}

		menuItemText = va_arg ( menuList, char * );
	}

	va_end ( menuList );
}


DMenuCmd::DMenuCmd ( const char *name, int active, Boolean quitOnSelect, DMenuCmdCallback callback,
					 void *clientData, char *menuLabelText,
					 int numMenuItems,
					 int charArraySize,
					 char *itemArray ) : NoUndoCmd ( name, active )
{
	_fixedString = FALSE;
	strncpy ( _name, name, sizeof ( _name ) - 1 );
	_callback = callback;
	_clientData = clientData;
	_menuItemList[0] = 0;
	_menuLabelText = menuLabelText;
	_numMenuItems = numMenuItems;
	_charArraySize = charArraySize;
	_pItemArray = itemArray;
	_quitOnSelect = quitOnSelect;
	_popupRealised = False;
	
	for ( unsigned int item = 0; item < _numMenuItems; item++ )
	{
		if ( ( strlen ( _menuItemList ) + strlen ( ( char * ) &_pItemArray[ item *
			charArraySize ] ) ) >=MENUSTORAGESPACE )
		{
			char msg[1024];
			snprintf ( msg, sizeof ( msg ) , "Menu item space required exceeds storage space\n\
%d allocated in DMenuCmd.h", MENUSTORAGESPACE );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
		}
		else
		{
			strncat ( _menuItemList, ( char * ) &_pItemArray[ item * charArraySize ], sizeof ( _menuItemList ) - strlen ( _menuItemList ) - 1 );
			strncat ( _menuItemList, ",", sizeof ( _menuItemList ) - strlen ( _menuItemList ) - 1 );
		}
	}
}


DMenuCmd::~DMenuCmd()
{
	if ( _popupRealised == True )
		quit();
}


void DMenuCmd::doCommand()
{
	unsigned int n;
	Atom WM_DELETE_WINDOW;

	_popup = XtCreatePopupShell ( _name, transientShellWidgetClass,
					application->baseWidget(), NULL, 0 );

	_menu = XtVaCreateManagedWidget ( "Menu", xmRowColumnWidgetClass,
						_popup,
						XmNorientation, XmVERTICAL,
						XmNpacking, XmPACK_TIGHT,
						XmNnumColumns, _numMenuItems+2,
						NULL );

	_menuLabel = XtCreateManagedWidget ( _menuLabelText, xmLabelWidgetClass, _menu, NULL, 0 );

	if ( _fixedString == TRUE )
	{
		unsigned int start = 0;
		unsigned int end = 0;
		unsigned int ptr;

		arrayOfPointersToMemBlocks = new menuWidgetSet *[_numMenuItems];

		for ( n = 1; n < ( _numMenuItems + 1 ); n++ )
		{
			arrayOfPointersToMemBlocks[n-1] = new menuWidgetSet;
			arrayOfPointersToMemBlocks[n-1]->n = n;
			arrayOfPointersToMemBlocks[n-1]->menu = this;

			while ( _menuItemList[end] != ',' )
				end++;

			char item [ end - start +1 ];
			ptr = 0;

			while ( start < end )
			{
				item[ptr] = _menuItemList[start];
				start++;
				ptr++;
			}

			item[ptr++] = 0;
			end++;
			start = end;

			_entry = XtVaCreateManagedWidget ( ( char * ) item,
								xmPushButtonWidgetClass,
								_menu, NULL );

			XtAddCallback(_entry, XmNactivateCallback, &DMenuCmd::menuEntrySelectCallback, ( XtPointer ) arrayOfPointersToMemBlocks[n-1] );
		}
	}
	else
	{
		arrayOfPointersToMemBlocks = new menuWidgetSet * [ _numMenuItems ];

		for ( unsigned int item = 0; item < _numMenuItems; item++ )
		{
			arrayOfPointersToMemBlocks[ item ] = new menuWidgetSet;
			arrayOfPointersToMemBlocks[ item ]->n = item + 1;
			arrayOfPointersToMemBlocks[ item ]->menu = this;

			_entry = XtVaCreateManagedWidget ( ( char * ) &_pItemArray[ item * _charArraySize ],
								xmPushButtonWidgetClass,
								_menu, NULL );

			XtAddCallback(_entry, XmNactivateCallback,
					&DMenuCmd::menuEntrySelectCallback,
					( XtPointer ) arrayOfPointersToMemBlocks[item] );
		}
	}

	// cater for WM 'X' pressed
	WM_DELETE_WINDOW = XmInternAtom ( XtDisplay ( _popup ), "WM_DELETE_WINDOW", False );
	XmAddWMProtocols ( _popup, &WM_DELETE_WINDOW, 1 );
	XmAddWMProtocolCallback ( _popup, WM_DELETE_WINDOW, clientMessageProc, ( XtPointer ) this );

	_popupRealised = True;
	XtPopup ( _popup, ( XtGrabKind ) XtGrabNone );
	XtManageChild ( _popup );
}


void DMenuCmd::clientMessageProc ( Widget, XtPointer clientData, XtPointer cbs)
{

	DMenuCmd *obj = ( DMenuCmd * ) clientData;

	obj->menuEntrySelect ( obj->_numMenuItems );
}


void DMenuCmd::menuEntrySelectCallback( Widget, XtPointer clientData, XtPointer )
{
	menuWidgetSet *widgetPak = ( menuWidgetSet * ) clientData;
	DMenuCmd *managedMenu = widgetPak->menu;

	managedMenu->menuEntrySelect ( widgetPak->n  );	
}

// class member functions for callback functions

void DMenuCmd::menuEntrySelect ( unsigned int itemSelected )
{
	unsigned int item = 0;
	int ptr = 0;
	unsigned int n = 0;
	char label [ _charArraySize + 1 ];

	for ( n = 0; n < strlen ( _menuItemList ); n++ )
	{
		if ( _menuItemList[n] == ',' )
			item++;
			
		if ( item == ( itemSelected - 1 ) )
		{
			ptr = n;
			n = strlen ( _menuItemList );
		}
	}

	if ( ptr > 0 )
		ptr++;

	n = 0;

	while ( _menuItemList[ptr] != ',' )
	{
		label[n] = _menuItemList[ptr];
		n++;
		ptr++;
	}
	
	label[n] = 0;
	setSelectedItem ( itemSelected );

	if ( _callback )
		_callback ( _clientData, itemSelected, ( char * ) &label );	// also report back on 'quit selected'

	if ( ( itemSelected == _numMenuItems ) || ( _quitOnSelect == True ) || ( itemSelected == 0 ) )
		quit();
}


void DMenuCmd::quit()
{
	if ( _numMenuItems > 0 )
	{
		for ( unsigned int count = 0; count < _numMenuItems; count++ )
			delete arrayOfPointersToMemBlocks[ count ];

		delete[] arrayOfPointersToMemBlocks;
		arrayOfPointersToMemBlocks = 0;
		_numMenuItems = 0;
		_menuItemList[0] = 0;
	}

	if ( _popupRealised == True )
	{
		_popupRealised = False;
		XtDestroyWidget ( _popup );
	}
}


void DMenuCmd::setTitle ( char *title )
{
	strncpy ( _menuTitle, title, sizeof ( _menuTitle ) );
	_menuTitle [ sizeof ( _menuTitle ) - 1 ] = 0;
	
	XmString xmstr = XmStringCreateSimple ( _menuTitle );
	XtVaSetValues ( _menuLabel, XmNlabelString, xmstr, NULL );
	XmStringFree ( xmstr );
}
